import json
from copy import deepcopy

def generate_config_h(config, output_file):
    with open(output_file, 'w') as f:
        f.write("// This file is generated by cfg.py\n")
        f.write("// Do not modify this file\n")
        f.write("#ifndef CONFIG_H\n")
        f.write("#define CONFIG_H\n\n")

        mode = config.get("Mode", "Debug")
        mode_config = config.get(mode, {})

        for key, value in mode_config.items():
            if isinstance(value, bool) and value:
                macro_name = f"Debug{key}"
                f.write(f"#define {macro_name}\n")

        system_config = config.get("System", {})
        for key, value in system_config.items():
            if isinstance(value, bool) and value:
                macro_name = f"System{key}"
                f.write(f"#define {macro_name}\n")

        f.write("\n#endif // CONFIG_H\n")

def load_config(config_path):
    with open(config_path, 'r') as f:
        config = json.load(f)
    return config

if __name__ == "__main__":
    config = load_config("config/config.json")
    generate_config_h(config, "kernel/include/config.h")