#ifndef TRAP_H
#define TRAP_H
#include "type.h"
#include "assert.h"
#include "printk.h"
#include "riscv.h"
#include "timer.h"

#define read_csr(reg) ({ unsigned long __tmp; \
    asm volatile ("csrr %0, " #reg : "=r"(__tmp)); \
    __tmp; })
#define write_csr(reg, val) ({ \
    asm volatile ("csrw " #reg ", %0" :: "rK"(val)); })


typedef enum interrupt_t{
    INTERRUPT_SOFTWARE = 1,
    INTERRUPT_TIMER = 5,
    INTERRUPT_EXTERNAL = 9,
} interrupt_t;

typedef enum exception_t{
    EXECPTION_INST_ADDR_MISALIGNED = 0,
    EXECPTION_INST_ACCESS_FAULT = 1,
    EXECPTION_ILLEGAL_INST = 2,
    EXECPTION_BREAKPOINT = 3,
    EXECPTION_LOAD_ADDR_MISALIGNED = 4,
    EXECPTION_LOAD_ACCESS_FAULT = 5,
    EXECPTION_STORE_AMO_ADDR_MISALIGNED = 6,
    EXECPTION_STORE_AMO_ACCESS_FAULT = 7,
    EXECPTION_ENV_CALL_FROM_U = 8,
    EXECPTION_ENV_CALL_FROM_S = 9,
    EXCEPTION_ENV_CALL_FROM_M = 11,
    EXCEPTION_INST_PAGE_FAULT = 12,
    EXCEPTION_LOAD_PAGE_FAULT = 13,
    EXCEPTION_STORE_AMO_PAGE_FAULT = 15,
} exception_t;

#define INTERRUPT_FLAG 0x8000000000000000
#define INTERRUPT_CAUSE(cause) (interrupt_t)(cause & 0xff)


#define SSTATUS_SIE 0x2
#define SSTATUS_SPIE 0x20
#define SSTATUS_SPP 0x100

#define SIE_SEIE 0x80
#define SIE_STIE 0x20
#define SIE_SSIE 0x2

void init_trap(void);
void trap_handler(void);
void interrupt_handler(interrupt_t cause, uint64_t mepc, uint64_t sstatus);
void exception_handler(exception_t cause, uint64_t mepc, uint64_t sstatus);
void handle_timer_interrupt(void);
#endif