#include "include/trap.h"
#include "include/assert.h"
#include "include/printk.h"

extern void trap_vector();
void init_trap(void) {
    write_csr(stvec, (uint64_t)trap_vector);
    write_csr(sstatus, read_csr(sstatus) | SSTATUS_SIE);
    write_csr(sie, SIE_SEIE | SIE_STIE | SIE_SSIE);
}

void trap_handler(void) {
    uint64_t sepc = read_csr(sepc);
    uint64_t sstatus = read_csr(sstatus);
    uint64_t scause = read_csr(scause);
    uint64_t stval = read_csr(stval);
    uint64_t sscratch = read_csr(sscratch);

    printk("Trap: sepc=%lx, sstatus=%lx, scause=%lx, stval=%lx, sscratch=%lx\n", 
            sepc, sstatus, scause, stval, sscratch);

    Assert(sstatus & SSTATUS_SPP, "Trap from user mode!");
    Assert(sstatus & SSTATUS_SPIE, "Interrupts are disabled!");
    Assert(!(sstatus & SSTATUS_SIE), "Interrupts are enabled when interrupt happens!");

    if (scause & INTERRUPT_FLAG)
        interrupt_handler(INTERRUPT_CAUSE(scause), sepc, sstatus);
    else
        exception_handler(scause, sepc, sstatus);
}



void interrupt_handler(interrupt_t cause, uint64_t sepc, uint64_t sstatus) {
    switch (cause) {
        case INTERRUPT_TIMER:
            handle_timer_interrupt();
            write_csr(sstatus, read_csr(sstatus) | SSTATUS_SPIE);
            write_csr(sepc, sepc);
            break;
        default:
            panic("Unknown interrupt!");
            break;
    }
}

void exception_handler(exception_t cause, uint64_t sepc, uint64_t sstatus) {
    switch (cause) {
    default:
        break;
    }
}

void handle_timer_interrupt(void) {
    static int count = 0;
    set_next_timer();
    printk("Timer interrupt %d!\n", count++);
}